"""

Main page: https://www.nordiska.uu.se/forskn/samnord.htm/?languageId=1
Data archive: www.runforum.nordiska.uu.se/filer/srd2014.zip
License: https://opendatacommons.org/licenses/odbl/1.0/
loggbok: http://www.runforum.nordiska.uu.se/filer/loggbok_2008-2014.pdf
"""

import os
import time
import json
from lxml import html
import requests
import zipfile
import codecs

from runesanalyzer.data import sweden_runic_inscription_filename
from runesanalyzer.reader import read_rundata

__author__ = ["Clément Besnier <clemsciences@aol.com>", ]


def retrieve_scandinavian_runic_text_database():
    """

    :return:
    """
    data = requests.get("http://www.runforum.nordiska.uu.se/filer/srd2014.zip")
    with open("data.zip", "wb") as f:
        f.write(data.content)

    zip_ref = zipfile.ZipFile("data.zip", 'r')
    zip_ref.extractall("data_runes")
    zip_ref.close()


def read_fornspr():
    return {}


def read_fornnsprx():
    return {}


def read_fvn():
    return {}


def read_fvnx():
    return {}


def read_litt():
    return {}


def read_runtext():
    return {}


def read_runtextx():
    return {}


def retrieve_sweden_runic_inscription():
    site = "http://runes.verbix.com/index.html"
    rrunes = requests.get(site)
    tree = html.fromstring(rrunes.text)
    links = [i.get("href") for i in tree.xpath("//a") if "vikingage" in i.get("href")]
    runes = {}
    for link in links:
        page = requests.get("http://runes.verbix.com/"+link)
        tree = html.fromstring(page.text)
        rune_blocks = tree.xpath('//p[@class="rrune"]')
        print(len(rune_blocks))
        content = [rb.text for rb in rune_blocks]
        runes[link] = content
        print(content)
        time.sleep(1)
    with open("sweden_runes.json", "w") as f:
        json.dump(runes, f)

    with open("sweden_runes.json", "r") as f:
        loaded_runes = json.load(f)

    print(loaded_runes.keys())


if __name__ == "__main__":
    # retrieve_sweden_runic_inscription()
    # sweden_runic_inscription_filename()
    # retrieve_scandinavian_runic_text_database()
    print(read_rundata()[:10])



"""

"""

import unittest
from runesanalyzer import data

__author__ = ["Clément Besnier <clemsciences@aol.com>", ]


class Tests(unittest.TestCase):
    def test_elder_futhark(self):
        self.assertListEqual([], data.ELDER_FUTHARK)




import os
import codecs
import csv

from runesanalyzer import PACKDIR


__author__ = ["Clément Besnier <clemsciences@aol.com>", ]

data_directory = os.path.join(PACKDIR, "data_runes")


def load_rundata():
    with codecs.open(os.path.join(data_directory, "RUNTEXTX"), "r", encoding="utf-8") as f:
        for line in f:
            yield line


def read_rundata():
    inscriptions = []
    with codecs.open(os.path.join(data_directory, "RUNTEXT"), "r", encoding="utf-8") as f:
        for line in f:
            # print(line)
            if line[0] != "!":
                l_line = line.strip().split(" ")
                if len(l_line) > 2:
                    inscriptions.append({"location": l_line[0], "period": l_line[1], "content": " ".join(l_line[2:])})
    return inscriptions


def load_urnodisk():
    i = 0
    for line in load_rundata():
        if line[0] != "!":
            l_line = line.strip().split(" ")
            if len(l_line) > 2 and l_line[2] == "U":
                print(" ".join(l_line[3:]))
                i += 1
    print(i)


def load_metadata():
    with codecs.open(os.path.join(data_directory, "RUNDATA.csv"), encoding="utf-8") as f:
        rundata_reader = csv.DictReader(f, dialect="excel-tab")
        print(rundata_reader.fieldnames)
        for line in rundata_reader:
            yield line


def get_urnordisk_inskritfter_datum():
    signa = []
    for line in load_metadata():
        if line["Period/Datering"].startswith("U"):
            signa.append(line["Signum"])
    for line in load_rundata():
        if line[0] != "!":
            l_line = line.strip().split(" ")
            if l_line[0] in signa:
                if len(l_line) > 2:
                    print(" ".join(l_line[3:]))


if __name__ == "__main__":
    # load_urnodisk()
    get_urnordisk_inskritfter_datum()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# For a fully annotated version of this file and what it does, see
# https://github.com/pypa/sampleproject/blob/master/setup.py

# To upload this file to PyPI you must build it then upload it:
# python setup.py sdist bdist_wheel  # build in 'dist' folder
# python-m twine upload dist/*  # 'twine' must be installed: 'pip install twine'

import io
import os
from setuptools import find_packages, setup

DEPENDENCIES = []
EXCLUDE_FROM_PACKAGES = []
CURDIR = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(CURDIR, "README.md"), "r", encoding="utf-8") as f:
    README = f.read()

setup(
    name='runesanalyzer',
    version='1.0.4',
    packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    include_package_data=True,
    url='https://github.com/clemsciences/runes-analyzer',
    license="License :: OSI Approved :: MIT License",
    author='Clément Besnier',
    author_email='clemsciences@gmail.com',
    description='Gathers different kinds of runic inscriptions',
    long_description=README,
    long_description_content_type="text/markdown",
    install_requires=DEPENDENCIES,
    keywords=["runes", "corpus"],
    zip_safe=True,
    python_requires=">=3.6",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        "Topic :: Text Processing",
    ],

)



"""
Sources:
- Viking Language 1 by Jessie L. Byock 2013


Unicode: 16A0–16FF
Swedish runic inscriptions
http://runes.verbix.com/index.html
http://runes.verbix.com/vikingage/Uppland.html

"""
import json
from runesanalyzer.runes import RunicAlphabetName, Rune, Transcriber

__author__ = ["Clément Besnier <clemsciences@aol.com>", ]


# ᚠ ᚢ ᚦ ᚨ ᚲ ᚱ ᚷ ᚹ ᚼ ᚾ ᛁ ᛃ ᛇ ᛈ ᛉ ᛊ ᛏ ᛒ ᛖ ᛗ ᛚ ᛜ ᛟ ᛞ
ELDER_FUTHARK = [
    Rune(RunicAlphabetName.elder_futhark, "\u16A0", "f", "f", "fehu"),
    Rune(RunicAlphabetName.elder_futhark, "\u16A2", "u", "u", "uruz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16A6", "θ", "þ", "þuriaz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16A8", "a", "a", "ansuz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16B1", "r", "r", "raido"),
    Rune(RunicAlphabetName.elder_futhark, "\u16B2", "k", "k", "kaunan"),
    Rune(RunicAlphabetName.elder_futhark, "\u16B7", "g", "g", "gyfu"),
    Rune(RunicAlphabetName.elder_futhark, "\u16B9", "w", "w", "wynn"),

    Rune(RunicAlphabetName.elder_futhark, "\u16BA", "h", "h", "haglaz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16BE", "n", "n", "naudiz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16C1", "i", "i", "isaz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16C3", "j", "j", "jeran"),
    Rune(RunicAlphabetName.elder_futhark, "\u16C7", "æ", "E", "eiwaz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16C8", "p", "p", "peorð"),
    Rune(RunicAlphabetName.elder_futhark, "\u16C9", "ʀ", "R", "algiz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16CA", "s", "s", "sowilo"),

    Rune(RunicAlphabetName.elder_futhark, "\u16CF", "t", "t", "tiwaz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16D2", "b", "b", "berkanan"),
    Rune(RunicAlphabetName.elder_futhark, "\u16D6", "e", "e", "ehwaz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16D7", "m", "m", "mannaz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16DA", "l", "l", "laguz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16DC", "ŋ", "ng", "ingwaz"),
    Rune(RunicAlphabetName.elder_futhark, "\u16DF", "ø", "œ", "odal"),
    Rune(RunicAlphabetName.elder_futhark, "\u16DE", "d", "d", "dagaz"),
]


# ᚠ ᚢ ᚦ ᚭ ᚱ ᚴ ᚼ ᚾ ᛁ ᛅ ᛋ ᛏ ᛒ ᛖ ᛘ ᛚ ᛦ
YOUNGER_FUTHARK = [
    Rune(RunicAlphabetName.younger_futhark, "\u16A0", "f", "f", "fehu"),
    Rune(RunicAlphabetName.younger_futhark, "\u16A2", "u", "u", "uruz"),
    Rune(RunicAlphabetName.younger_futhark, "\u16A6", "θ", "þ", "þuriaz"),
    Rune(RunicAlphabetName.younger_futhark, "\u16AD", "a", "a", "ansuz"),
    Rune(RunicAlphabetName.younger_futhark, "\u16B1", "r", "r", "raido"),
    Rune(RunicAlphabetName.younger_futhark, "\u16B4", "k", "k", "kaunan"),

    Rune(RunicAlphabetName.younger_futhark, "\u16BC", "h", "h", "haglaz"),
    Rune(RunicAlphabetName.younger_futhark, "\u16BE", "n", "n", "naudiz"),
    Rune(RunicAlphabetName.younger_futhark, "\u16C1", "i", "i", "isaz"),
    Rune(RunicAlphabetName.younger_futhark, "\u16C5", "a", "a", "jeran"),
    Rune(RunicAlphabetName.younger_futhark, "\u16CB", "s", "s", "sowilo"),

    Rune(RunicAlphabetName.younger_futhark, "\u16CF", "t", "t", "tiwaz"),
    Rune(RunicAlphabetName.younger_futhark, "\u16D2", "b", "b", "berkanan"),
    Rune(RunicAlphabetName.younger_futhark, "\u16D6", "e", "e", "ehwaz"),
    Rune(RunicAlphabetName.younger_futhark, "\u16D8", "m", "m", "mannaz"),  # also \u16D9
    Rune(RunicAlphabetName.younger_futhark, "\u16DA", "l", "l", "laguz"),
    Rune(RunicAlphabetName.younger_futhark, "\u16E6", "r", "R", "algiz")
]


# ᚠ ᚢ ᚦ ᚭ ᚱ ᚴ ᚽ ᚿ ᛁ ᛅ ᛌ ᛐ ᛓ ᛖ ᛙ ᛚ ᛧ
SHORT_TWIG_YOUNGER_FUTHARK = [
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16A0", "f", "f", "fehu"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16A2", "u", "u", "uruz"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16A6", "θ", "þ", "þuriaz"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16AD", "a", "a", "ansuz"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16B1", "r", "r", "raido"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16B4", "k", "k", "kaunan"),

    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16BD", "h", "h", "haglaz"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16BF", "n", "n", "naudiz"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16C1", "i", "i", "isaz"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16C5", "a", "a", "jeran"),  # not good
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16CC", "s", "s", "sowilo"),

    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16D0", "t", "t", "tiwaz"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16D3", "b", "b", "berkanan"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16D6", "e", "e", "ehwaz"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16D9", "m", "m", "mannaz"),  # also \u16D9
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16DA", "l", "l", "laguz"),
    Rune(RunicAlphabetName.short_twig_younger_futhark, "\u16E7", "r", "R", "algiz"),
]


# https://fr.wikipedia.org/wiki/Petite_pierre_de_Jelling
little_jelling_stone = "᛬ᚴᚢᚱᛘᛦ᛬ᚴᚢᚾᚢᚴᛦ᛬ᚴ(ᛅᚱ)ᚦᛁ᛬ᚴᚢᛒᛚ᛬ᚦᚢᛋᛁ᛬ᛅ(ᚠᛏ)᛬ᚦᚢᚱᚢᛁ᛬ᚴᚢᚾᚢ᛬ᛋᛁᚾᛅ᛬ᛏᛅᚾᛘᛅᚱᚴᛅᛦ᛬ᛒᚢᛏ᛬"
big_jelling_stone = "ᚼᛅᚱᛅᛚᛏᚱ᛬ᚴᚢᚾᚢᚴᛦ᛬ᛒᛅᚦ᛬ᚴᛅᚢᚱᚢᛅ ᚴᚢᛒᛚ᛬ᚦᛅᚢᛋᛁ᛬ᛅᚠᛏ᛬ᚴᚢᚱᛘ ᚠᛅᚦᚢᚱ ᛋᛁᚾ ᛅᚢᚴ ᛅᚠᛏ᛬ᚦᚭᚢᚱᚢᛁ᛬ᛘᚢᚦᚢᚱ᛬ᛋᛁᚾᛅ᛬ᛋᛅ " \
                    "ᚼᛅᚱᛅᛚᛏᚱ(᛬)ᛁᛅᛋ᛬ᛋᚭᛦ᛫ᚢᛅᚾ᛫ᛏᛅᚾᛘᛅᚢᚱᚴ\nᛅᛚᛅ᛫ᛅᚢᚴ᛫ᚾᚢᚱᚢᛁᚴ\n᛫ᛅᚢᚴ᛫ᛏ(ᛅ)ᚾᛁ(᛫ᚴᛅᚱᚦᛁ᛫)ᚴᚱᛁᛋᛏᚾᚭ"

ramsund_runestone = "ᛋᛁᚱᚦ᛬ᚴᛁᛅᚱᚦᛁ᛬ᛒᚢᚦ᛬ᚦᚭᛋᛁ᛬ᛘᚢᚦᛁᛦ᛬ᛅᛚᚱᛁᚴᛋ᛬ᛏᚢᛏᛁᛦ᛬ᚢᚱᛘᛋ᛬ᚠᚢᚱ᛬ᛋᛅᛚᚢ᛬ᚼᛅᛚᚢ᛬ᚼᚢᛚᛘᚴᛁᚱᛋ᛬ᚠᛅᚦᚢᚱ᛬ᛋᚢᚴᚱᚢᚦᛅᚱ᛬ᛒᚢᛅᛏᛅ᛬ᛋᛁᛋ"

# The Ed (Boulder) Inscription from Uppland, Sweden
# http://runes.verbix.com/vikingage/Uppland.html
ed_inscription = "ᚱᛅᚼᚾᚢᛅᛚᛏᚱ ᛫ ᛚᛁᛏ ᛫ ᚱᛁᛋᛏᛅ ᛫ ᚱᚢᚾᛅᚱ ᛫ ᚽᚡᛦ ᛫ ᚡᛅᛋᛏᚢᛁ ᛫ ᛘᚬᚦᚢᚱ ᛫ ᛋᛁᚾᛅ ᛫ ᚬᚾᚽᛘᛋ ᛫ ᛏᚬᛏᛦ ᛫ ᛏᚬ ᛁ ᛫ " \
                 "ᛅᛁᚦᛁ ᛫ ᚴᚢᚦ ᚼᛁᛅᛚᛒᛁ ᛫ ᛅᚾᛏ ᛫ ᚼᚽᚾᛅ ᛫ " \
                 "§B ᚱᚢᚾᛅ ᛫ ᚱᛁᛋᛏᛅ ᛫ ᛚᛁᛏ ᛫ ᚱᛅᚼᚾᚢᛅᛚᛏᚱ ᛫ ᚼᚢᛅᚱ ᛅ × ᚵᚱᛁᚴᛚᛅᚾᛏᛁ ᛫ ᚢᛅᛋ ᛫ ᛚᛁᛋ ᛫ ᚡᚬᚱᚢᚾᚴᛁ ᛫"

# https://en.wikipedia.org/wiki/Uppland_Runic_Inscription_448
u448 = "ᚴᚢᛚ᛫ᛅᚢᚴ᛫ᛒᛁᚢᚱᚾ᛫ᛚᛁᛏᚢ᛫ᚱᛅᛁᛋᛅ᛫ᛋᛏᛅᛁᚾ᛫ᛂᚠᛏᛁᛦ᛫ᚦᚢᚱᛋᛏᛅᛁᚾ᛫ᚠᛅᚦᚢᚱ"


# https://en.wikipedia.org/wiki/Uppland_Runic_Inscription_92
u92 = "ᚴᚾᚢᛏᚱ ' ᛁ ᚢᛁᚴ'ᚼᚢᛋᚢᛘ ' ᛚᛁᛏ ' ᛋᛏᛅᛁᚾ ' ᚱᛁᛏᛅ ' ᚢᚴ ' ᛒᚱᚬ ' ᚴᛁᚱᛅ ᛫ ᛁᚡᛏᛁᛦ ' ᚡᛅᚦᚢᚱ ᚢᚴ ᛫ ᛘᚬᚦᚬᚱ ᛫ ᚢᚴ ᛫" \
      " ᛒᚱᚤᚦᚱ ᛫ ᛋᛁᚾᛅ ᛫ ᚢᚴ ᛫ ᛋᚢᛋᛏᚢᚱ"


sweden_runic_inscription_filename = "sweden_runes.json"


def read_sweden_runes():
    with open(sweden_runic_inscription_filename, "r") as f:
        return json.load(f)


if __name__ == "__main__":
    print(" ".join(Rune.display_runes(ELDER_FUTHARK)))
    print(" ".join(Rune.display_runes(YOUNGER_FUTHARK)))
    print(" ".join(Rune.display_runes(SHORT_TWIG_YOUNGER_FUTHARK)))

    # https://sv.wikipedia.org/wiki/Upplands_runinskrifter_937
    inscription = "ᚦᛁᛅᚴᚾ᛫ᛅᚢᚴ᛫ᚴᚢᚾᛅᚱ᛫ᚱᛅᛁᛋᛏᚢ᛫ᛋᛏᛅᛁᚾᛅ ᛅᚠᛏᛁᛦ᛫ᚢᛅᚱ᛫ᛒᚱᚢᚦᚢᚱ᛫ᛋᛁᚾ"
    print(inscription)
    print(Transcriber.transcribe(inscription, YOUNGER_FUTHARK))
    print(little_jelling_stone)
    print(Transcriber.transcribe(little_jelling_stone, YOUNGER_FUTHARK))
    print(big_jelling_stone)
    print(Transcriber.transcribe(big_jelling_stone, YOUNGER_FUTHARK))
    print(ramsund_runestone)
    print(Transcriber.transcribe(ramsund_runestone, YOUNGER_FUTHARK))



"""

"""


from enum import auto
from runesanalyzer.utils import AutoName


__author__ = ["Clément Besnier <clemsciences@aol.com>", ]

POINT = "᛫"
SEMI_COLUMN = "\u16EC"


class RunicAlphabetName(AutoName):
    elder_futhark = auto()
    younger_futhark = auto()
    short_twig_younger_futhark = auto()


class Rune:
    def __init__(self, runic_alphabet: RunicAlphabetName, form: str, sound: str, transcription: str, name: str):
        self.runic_alphabet = runic_alphabet
        self.form = form
        self.sound = sound
        self.transcription = transcription
        self.name = name

    @staticmethod
    def display_runes(runic_alphabet: list):
        return [rune.form for rune in runic_alphabet]

    @staticmethod
    def from_form_to_transcription(form: str, runic_alphabet: list):
        d_form_transcription = {rune.form: rune.transcription for rune in runic_alphabet}
        return d_form_transcription[form]

    def __repr__(self):
        return self.form

    def __str__(self):
        return self.form

    def __eq__(self, other):
        return self.form == other


class Transcriber:
    def __init__(self):
        pass

    @staticmethod
    def from_form_to_transcription(runic_alphabet: list):
        return {rune.form: rune.transcription for rune in runic_alphabet}

    @staticmethod
    def transcribe(rune_sentence: str, runic_alphabet: list):
        res = []
        d_form_transcription = Transcriber.from_form_to_transcription(runic_alphabet)
        for c in rune_sentence:
            if c in runic_alphabet:
                res.append(d_form_transcription[c])
            elif c in "()":
                res.append(c)
            else:
                res.append(POINT)
        return "".join(res)
